/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.samples;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import ro.sync.ecss.extensions.api.component.AuthorComponentException;
import ro.sync.ecss.extensions.api.component.AuthorComponentFactory;

/**
 * Demonstrates the use of the author component as an applet.
 */
@SuppressWarnings("serial")
public class AuthorComponentSampleApplet extends JApplet {
	
	/**
	 * The sample component wrapper
	 */
	private AuthorComponentSample sample;

	/**
	 * @see java.applet.Applet#init()
	 */
	@Override
	public void init() {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					try {
						// No restrictions to this signed applet 
					   //If you encounter any security problems running the applet uncomment the line below: 
//						System.setSecurityManager(null);
						//THIS IS THE WAY IN WHICH YOU CAN REGISTER YOUR OWN PROTOCOL HANDLER TO THE JVM.
						//THEN YOU CAN OPEN YOUR CUSTOM URLs IN THE APPLET AND THE APPLET WILL USE YOUR HANDLER
//						URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {
//							public URLStreamHandler createURLStreamHandler(String protocol) {
//								if("cms".equals(protocol)) {
//									return new CMSHandler();
//								}
//								if("http".equals(protocol) || "https".equals(protocol)) {
//									return new URLStreamHandler() {
//										@Override
//										protected URLConnection openConnection(URL u) throws IOException {
//											URLConnection connection = new HttpURLConnection(u, null);
//											if(!u.toString().endsWith(".jar")) {
//												//Do not cache HTTP resources other than JARS
//												//By default the Java HTTP connection caches content for 
//												//all URLs so if one URL is modified and then re-loaded in the
//												//applet the applet will show the old content.
//												connection.setDefaultUseCaches(false);
//											}
//											return connection;
//										}
//									};
//								}
//								return null;
//							}
//						});
						// System look and feel
						UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

						// A gray to match the web page.
						Color gray = new Color(240,240,240);
						UIManager.put("Panel.background", gray);
						UIManager.put("ToolBar.background", gray);
						UIManager.put("SplitPaneDivider.background", gray);
						UIManager.put("ScrollBar.background", gray);

						//The document base
						URL docBase = getDocumentBase();
						System.out.println("[OXYGEN] Doc base is " + docBase);
						startAppletInit();
					} catch(Throwable t) {
						t.printStackTrace();
					}
				}});
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		super.init();
	}

	/**
	 * @see java.applet.Applet#stop()
	 */
	@Override
	public void stop() {
		super.stop();
	}

	/**
	 * @see java.applet.Applet#start()
	 */
	@Override
	public void start() {
		super.start();
	}

	/**
	 * Getter used from JavaScript
	 * @return The author component sample. Can be <code>null</code> if the 
	 * AuthorComponent did not initialized propertly, for instance a licens problem.
	 */
	public AuthorComponentSample getAuthorComponentSample() {
		return sample;
	}

	/**
	 * Setter used from JavaScript
	 * 
	 * @param width The new applet width
	 * @param height The new applet height
	 */
	public void changeSizeTo(final int width, final int height) {
		AuthorComponentSample.invokeOnAWT(new Runnable() {
			public void run() {
				AuthorComponentSampleApplet.super
				.setPreferredSize(new Dimension(width, height));
				AuthorComponentSampleApplet.super.setSize(width, height);
				AuthorComponentSampleApplet.super.validate();
			}
		});
	}
	
	
	/**
	 * Initialize the main applet components on a thread.
	 */
	private void startAppletInit() {
		final JPanel progressPanel = new JPanel(new GridLayout(2, 1));
		progressPanel.setBorder(new EmptyBorder(5, 15, 0, 15));
		progressPanel.add(new JLabel("Initializing..."));
		JProgressBar progressBar = new JProgressBar();
		progressBar.setIndeterminate(true);
		progressPanel.add(progressBar);
		add(progressPanel, BorderLayout.NORTH);
		new Thread() {
			public void run() {
				try {
					// Create the array of ZIPPED document type URLs 
					List<URL> frameworksURLs = new ArrayList<URL>();
					ClassLoader classLoader = AuthorComponentSampleApplet.class.getClassLoader();
					
					// Get the array of ZIPs from the applet
					URL frameworks = classLoader.getResource("frameworks.zip");
					if (frameworks != null) {
						frameworksURLs.add(frameworks);
					}
					
					// Get the ZIP pointing to fixed options if available.
					URL optionsZipURL = classLoader.getResource("options.zip");
					
					// The applet ID
					String appletID = getParameter("AppletID");
					
					String addToolbar = getParameter("addToolbar");
					String addHelperViews = getParameter("addHelperViews");
					String addBreadCrumb = getParameter("addBreadCrumb");
					
					
					// Check if the applet should show the "Open" action
					boolean showOpenLocal = false;
					
					URL documentBase = getDocumentBase();
					String query = documentBase.getQuery();
					if (query != null && query.contains("open_local")) {
						showOpenLocal = true;
					}
					
					boolean showToolbar = true;
					if ("false".equals(addToolbar)) {
					  showToolbar = false;
					}
					
					boolean showViews = true;
          if ("false".equals(addHelperViews)) {
            showViews = false;
          }
          
          boolean showBreadCrumb = true;
          if ("false".equals(addBreadCrumb)) {
            showBreadCrumb = false;
          }
					
					//Enable debug from JNLP or HTML.
					initLogger();
					
					// Create the sample panel
					sample = new AuthorComponentSample(
							frameworksURLs.toArray(new URL[0]), optionsZipURL, getCodeBase(), appletID, 
							showToolbar, showViews, showBreadCrumb, showOpenLocal);
					if (showToolbar) {
					  sample.setBorder(BorderFactory.createEmptyBorder(15, 0, 0, 0));
					}
					//No side views by default for the applet.
					if (showViews) {
					  sample.setVisibleSideView("outline", false);
					  sample.setVisibleSideView("attributes", false);
					  sample.setVisibleSideView("elements", false);
					  sample.setVisibleSideView("review", false);
					  sample.setVisibleSideView("validationProblems", false);
					}
					
					//Signal to Javascript loading has ended.
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							try {
								//Load default content if necessary
								String loadDefaultContent = getParameter("loadDefaultContent");
								if("yes".equals(loadDefaultContent)) {
									sample
									.setDocumentContent(
											null,
											AuthorComponentSample.DEFAULT_DOC_CONTENT);
								} else {
									//Maybe load a remote URL from some place
									String loadLocationURL = getParameter("loadLocationURL");
									if(loadLocationURL != null && loadLocationURL.length() > 0) {
										try {
											URL toLoad = new URL(getDocumentBase(), loadLocationURL);
											sample.setDocumentContent(toLoad.toString(), null);
										} catch(MalformedURLException ex) {
											ex.printStackTrace();
											JOptionPane.showMessageDialog(AuthorComponentSampleApplet.this, ex.getMessage());
										} catch(AuthorComponentException ex) {
											JOptionPane.showMessageDialog(AuthorComponentSampleApplet.this, ex.getMessage());
											ex.printStackTrace();
										}
									} else {
										//Call the javascript method to let it know the applet has loaded.
										try {
											getAppletContext().showDocument(new URL("javascript:onLoad()"));
										} catch (MalformedURLException e) {
											JOptionPane.showMessageDialog(AuthorComponentSampleApplet.this, e.getMessage());
											//
										}
									}
								}
								//Remove the progress label
								remove(progressPanel);
								//Add the applet component
								add(sample);
								//Reconfigure
								invalidate();
								validate();
							} catch(Throwable t) {
								t.printStackTrace();
								JOptionPane.showMessageDialog(AuthorComponentSampleApplet.this, t.getMessage());
							}
						}
					});
				} catch (AuthorComponentException e) {
					JOptionPane.showMessageDialog(
							AuthorComponentSampleApplet.this,
							"AuthorComponent problem: " + e.getMessage(),
							"Cannot create author editor",
							JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
				} catch (Throwable e) {
					e.printStackTrace();
				}
			}

		}.start();
	}
	
	
	/**
	 * Init the logger from HTML or JNLP parameters 
	 */
	private void initLogger() {
		String SEVERITY = getParameter("loggerLevel");
		if("debug".equals(SEVERITY)) {
			Logger.getRootLogger().setLevel(Level.DEBUG);
		} else if("error".equals(SEVERITY)) {
			Logger.getRootLogger().setLevel(Level.ERROR);
		} else if("warn".equals(SEVERITY)) {
			Logger.getRootLogger().setLevel(Level.WARN);
		} else if("fatal".equals(SEVERITY)) {
			Logger.getRootLogger().setLevel(Level.FATAL);
		} else if("all".equals(SEVERITY)) {
			Logger.getRootLogger().setLevel(Level.ALL);
		}
	}
	
	/**
	 * @see java.applet.Applet#destroy()
	 */
	@Override
	public void destroy() {
		AuthorComponentFactory factory = AuthorComponentFactory.getInstance();
		factory.dispose();
		super.destroy();
	}
}
