package ro.sync.ecss.samples.reviewer;

import java.io.File;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.UIManager;

import org.apache.log4j.Logger;

/**
 * A stand alone simple reviewer application based on the AuthorComponent.
 * 
 * It allows the user to change the editing mode between: read-only, text editing only and 
 * full editing.
 * 
 * @author cristi_talau
 */
public class Main {
	/**
	 * The default document content.
	 */
	private final static String DEFAULT_DOC_CONTENT = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
			+ "<!DOCTYPE concept PUBLIC \"-//OASIS//DTD DITA Concept//EN\" \"http://docs.oasis-open.org/dita/v1.1/OS/dtd/concept.dtd\">\n"
			+ "<concept id=\"conceptId\">\n"
			+ "    <title>Winter Flowers</title>\n"
			+ "    <conbody>\n"
			+ "        <p>Winter is the season of cold weather. The season occurs during December - February in\n"
			+ "            Northern hemisphere . In the Southern hemisphere winter occurs during June - August. </p>\n"
			+ "        <p>Some of the flowers blooming in winter are: Acashia, Alstromeria, Amaryllis, Carnation,\n"
			+ "            Chrysanthemums, Cyclamen, Evergreens, Gerbera Daisy, Ginger, Helleborus, Holly berry,\n"
			+ "            Lily, Asiatic Lily, Casa Blanca Lily, Narcissus, Orchid, Pansy, Pepperberry, Phlox,\n"
			+ "            Protea, Queen Ann's Lace, Roses, Star of Bethlehem, Statice. </p>\n"
			+ "    </conbody>\n" + "</concept>\n" + "";

	/**
	 * Licensing a fixed license key (the license key string must be the same as
	 * the one received from oXygen, including start/end markers and line
	 * breaks)
	 * 
	 * IMPORTANT: THE AUTHOR COMPONENT CAN BE PROGRAMTICALLY LICENSED WITH A
	 * FIXED LICENSE KEY EITHER OF TYPE "EVALUATION" WHICH CAN BE USED DURING
	 * THE DEVELOPMENT PHASE OR OF TYPE SITE LICENSE. IF AN INVALID LICENSE KEY
	 * IS USED OR IF THE LICENSE KEY IS NULL, THE COMPONENT WILL REQUEST A
	 * LICENSE KEY FROM THE USER WHEN STARTED.
	 */
	private final static String licenseKey = 
			"------START-LICENSE-KEY------\n"
			+ "Registration_Name=Test\n" 
			+ "Company=Syncro\n" 
			+ "Category=Professional\n"
			+ "Component=Author-Component\n" 
			+ "Version=14\n" 
			+ "Number_of_Licenses=1\n"
			+ "Date=01-22-2013\n" 
			+ "EvaluationOnly=true\n"
			+ "SGN=MCwCFD4YoiV70tjVVbCxUACfGK050hnAAhQje6OGNG9xouwqxOIgtBtVmtYVfA\\=\\=\n"
			+ "-------END-LICENSE-KEY-------";

	/**
	 * Logger.
	 */
	static Logger logger = Logger.getLogger(Main.class.getName());

	/**
	 * Brings up a frame with the component sample.
	 * 
	 * @param args
	 *            ignored.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

			URL frameworksZipJarURL = new File("resources/frameworks.zip.jar").toURI().toURL();
			URL optionsZipJarURL = new File("resources/options.zip.jar").toURI().toURL();

			URL frameworksZipURL = new URL("jar:" + frameworksZipJarURL.toString() + "!/"
					+ "frameworks.zip");
			URL optionsZipURL = new URL("jar:" + optionsZipJarURL.toString() + "!/" + 
					"options.zip");

			JFrame frame = new JFrame("Author Component Sample Reviewer Application");
			AuthorComponentReviewerSample sample = new AuthorComponentReviewerSample(
			// Frameworks Zip
					new URL[] { frameworksZipURL },
					// Options Zip
					optionsZipURL,
					// The license key if it is a fixed license
					licenseKey);

			sample.setDocumentContent(null, DEFAULT_DOC_CONTENT);

			frame.setSize(1000, 700);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.getContentPane().add(sample);
			frame.setVisible(true);

		} catch (Exception e) {
			logger.error(e, e);
		}
	}

}
