/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.link;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Base class for custom elements locators used to locate 
 * an element based on a link. 
 * The source XML is parsed and notifications will be forwarded 
 * to ElementLocator objects in order for the references to be resolved.
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class ElementLocator {
  /**
   * The link to be used to identify the element.
   */
  protected final String link;

  /**
   * Constructor.
   * 
   * @param link The link to be used to identify the element.
   */
  public ElementLocator(String link) {
    this.link = link;
  }
  
  /**
   * Notification received when the beginning of an element has been encountered. 
   * This method is invoked at the beginning of every element in the 
   * XML document; an event will be fired for every <code>startElement</code> 
   * (even when the element is empty).
   * 
   * @param uri the namespace URI, or the empty string if the
   *        element has no namespace URI or if namespace
   *        processing is not being performed
   * @param localName the local name of the element
   * @param qName the qualified name of the element
   * @param atts an array with the attributes attached to the element. If
   *        there are no attributes, it shall be empty. 
   *        The attributes are represented as {@link Attr} objects.
   *        
   * @return <code>true</code> if the current element is indicated by the link.
   */
  public abstract boolean startElement(String uri, String localName, String qName, Attr[] atts) ;
  
  /**
   * Notification received when the end of an element has been encountered.
   *
   * This method is invoked at the end of every
   * element in the XML document; an event will be fired 
   * for every <code>endElement</code> (even when the element is empty).
   *
   * @param uri the namespace URI, or the empty string if the
   *        element has no namespace URI or if namespace
   *        processing is not being performed
   * @param localName the local name of the element
   * @param qName the qualified XML name of the element
   */
  public abstract void endElement(String uri, String localName, String qName);
}