/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.topic.table.simpletable;

import java.util.StringTokenizer;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorTableCellSpanProvider;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.commons.table.operations.DeleteColumnOperationBase;

/**
 * Operation used to delete a DITA simple table column.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DeleteColumnOperation extends DeleteColumnOperationBase implements SimpleTableConstants {
  
  /**
   * Constructor.
   */
  public DeleteColumnOperation() {
    super(new DITASimpleTableDocumentTypeHelper());
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.DeleteColumnOperationBase#updateTableColSpan(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.AuthorTableCellSpanProvider, ro.sync.ecss.extensions.api.node.AuthorElement, int, int)
   */
  @Override
  protected void updateTableColSpan(AuthorAccess authorAccess,
      AuthorTableCellSpanProvider spanProvider, AuthorElement cell, int colStartIndex,
      int colEndIndex) throws AuthorOperationException {
    // Unused.
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.DeleteColumnOperationBase#updateAppliableColWidthsNumber(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.node.AuthorElement, int)
   */
  @Override
  protected void updateAppliableColWidthsNumber(AuthorAccess authorAccess, AuthorElement tableElem,
      int deletedColumnIndex) {
    super.updateAppliableColWidthsNumber(authorAccess, tableElem, deletedColumnIndex);
    //EXM-25639 And also remove the span spec for it.
    AttrValue relColWidth = tableElem.getAttribute("relcolwidth");
    if(relColWidth != null) {
      //We need to update the current value to contain a "*" also for the newly added column.
      String currentValue = relColWidth.getValue();
      //Sometimes we might now be able to update the current value.
      boolean updatedValue = false;
      int valuesCounter = 0;
      StringBuilder newValue = new StringBuilder();
      StringTokenizer st = new StringTokenizer(currentValue);
      while(st.hasMoreElements()) {
        String token = st.nextToken();
        if(valuesCounter == deletedColumnIndex) {
          //Found the insertion place.
          updatedValue = true;
          valuesCounter ++;
          if(! st.hasMoreElements()) {
            //We have to remove a trailing space
            if(newValue.length() > 0) {
              newValue.deleteCharAt(newValue.length() - 1);
            }
          }
          //Skip this one
          continue;
        } else {
          //Just add the existing token.
          newValue.append(token);
          valuesCounter ++;
        }
        if(st.hasMoreElements()) {
          newValue.append(" ");
        }
      }
      if(updatedValue) {
        //We can set the new value
        authorAccess.getDocumentController().setAttribute(
            "relcolwidth", new AttrValue(newValue.toString()), tableElem);
      }
    }
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.DeleteColumnOperationBase#canDeleteColumn()
   */
  @Override
  protected boolean canDeleteColumn() {
    boolean canDeleteColumn = true;
    AttrValue classAttr = tableElem.getAttribute("class");
    if (classAttr != null && classAttr.getValue().contains(" task/choicetable ")) {
      // EXM-24849 Delete column on a choice table is not permitted.
      canDeleteColumn = false;
    }
    
    return canDeleteColumn;
  }
}